<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | Moblog Plugin                                                             |
// +---------------------------------------------------------------------------+
// | plugins/moblog/functions.inc                                              |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2009 by the following authors:                              |
// |                                                                           |
// | Authors: Yoshinori Tahara - taharaxp AT gmail DOT com                     |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+

/**
* @package Moblog
*/

if (strpos(strtolower($_SERVER['PHP_SELF']), 'functions.inc') !== false) {
    die ('This file can not be used on its own.');
}

/**
 * Language file include
 */
$plugin_path = $_CONF['path'] . 'plugins/moblog/';
$langfile = $plugin_path . 'language/' . $_CONF['language'] . '.php';

if (file_exists($langfile)) {
    require_once $langfile;
} else {
    require_once $plugin_path . 'language/english.php';
}

/**
 * Moblog plugin table(s)
 */
$_TABLES['moblog'] = $_DB_table_prefix . 'moblog';


$_MOBLOG_CONF = array();

// テンプレートのパス
$_MOBLOG_CONF['path_layout'] = $_CONF['path'] . 'plugins/moblog/templates/';

// ログイン要求
$_MOBLOG_CONF['login_required'] = false;

/**
* Check and see if we need to load the plugin configuration.
*/
require_once $_CONF['path_system'] . 'classes/config.class.php';
$plg_config = config::get_instance();
$temp = $plg_config->get_config('moblog');
if (is_array($temp)) {
    $_MOBLOG_CONF = array_merge($_MOBLOG_CONF, $temp);
}

// +---------------------------------------------------------------------------+
// | Geeklog Plugin API Implementation                                         |
// +---------------------------------------------------------------------------+

/**
* Return plugin entry for "Command and Control" (moderation.php)
*
* @return  array  Array containing (plugin name, admin url, url of plugin icon)
*
*/
function plugin_cclabel_moblog()
{
    global $_CONF, $LANG_MOBLOG_1;

    $retval = array();
    if (SEC_hasRights('moblog.admin')) {
        $retval = array($LANG_MOBLOG_1['plugin_name'],
            $_CONF['site_url'] . '/moblog/index.php',
            plugin_geticon_moblog());
    }

    return $retval;
}


/**
* Return plugin entry for the Admins Only block
*
* @return array Array containing (plugin name, plugin admin url, # of plugin items or '')
*
*/
function plugin_getadminoption_moblog()
{
    global $_CONF, $LANG_MOBLOG_1;

    if (SEC_hasRights('moblog.admin')) {
        return array($LANG_MOBLOG_1['plugin_name'],
            $_CONF['site_url'] . '/moblog/index.php', '');
    }
}


/**
* Returns the URL of the plugin's icon
*
* @return   string      URL of the icon
*
*/
function plugin_geticon_moblog()
{
    global $_CONF;

    return $_CONF['site_admin_url'] . '/plugins/moblog/images/moblog.png';
}

/**
 * Returns the current plugin code version
 *
 * @return string    plugin version
 */
function plugin_chkVersion_moblog()
{
    global $_CONF;

    require_once $_CONF['path'] . 'plugins/moblog/autoinstall.php';

    $inst_parms = plugin_autoinstall_moblog('moblog');

    return $inst_parms['info']['pi_version'];
}

/**
* Update the plugin
*
* @return   int     Number of message to display or true for success
*
*/
function plugin_upgrade_moblog()
{
    global $_CONF, $_TABLES;

    $installed_version = DB_getItem($_TABLES['plugins'], 'pi_version',
                                    "pi_name = 'moblog'");
    $code_version = plugin_chkVersion_moblog();
    if ($installed_version == $code_version) {
        // nothing to do
        return true;
    }

    require_once $_CONF['path'] . 'plugins/moblog/autoinstall.php';

    if (! plugin_compatible_with_this_version_moblog('moblog')) {
        return 3002;
    }

    // other update code goes here

    // update plugin version number
    $inst_parms = plugin_autoinstall_moblog('moblog');
    $pi_gl_version = $inst_parms['info']['pi_gl_version'];

    DB_query("UPDATE {$_TABLES['plugins']} "
           . "SET pi_version = '$code_version', pi_gl_version = '$pi_gl_version' "
           . "WHERE pi_name = 'moblog'");

    return true;
}

/**
* Automatic uninstall function for plugins
*
* @return   array   Plugin information
*
*/
function plugin_autouninstall_moblog()
{
    $out = array (
        /* give the name of the tables, without $_TABLES[] */
        'tables' => array('moblog'),
        /* give the full name of the group, as in the db */
        'groups' => array('Moblog Admin'),
        /* give the full name of the feature, as in the db */
        'features' => array('moblog.admin'),
        /* give the full name of the block, including 'phpblock_', etc */
        'php_blocks' => array(),
        /* give all vars with their name */
        'vars' => array()
    );

    return $out;
}

/**
* Returns the user option for this plugin
*
* @return   array   Array containing (plugin name, plugin user url, 0)
*
*/
function plugin_getuseroption_moblog()
{
    global $_CONF, $LANG_MOBLOG_1, $_MOBLOG_CONF;

    $url = $_CONF['site_url'] . '/moblog/userconf.php';
    return array ($LANG_MOBLOG_1['user_conf'], $url, '');
}

/**
* Callback function for the pseudo cron job 
*
*/
function plugin_runScheduledTask_moblog()
{
    global $_CONF, $_USER, $_TABLES, $_MOBLOG_CONF, $LANG_MOBLOG_1;

    if ($_MOBLOG_CONF['enable_cronjob']) {
        require $_CONF['path'] . 'plugins/moblog/moblog.php';
        MOBLOG_execProcess();
    }
}

/**
* When a user is deleted, update the database
*
* @param    int     $uid    User id of deleted user
* @return   void
*
*/
function plugin_user_delete_moblog ($uid)
{
    global $_TABLES, $_MOBLOG_CONF;

    if (DB_count ($_TABLES['moblog'], 'uid', $uid) == 0) {
        return;
    }

    DB_delete($_TABLES['moblog'], 'uid', $uid);
}

/**
* When a user is created, update the database
*
* @param    int     $uid    User id of deleted user
* @return   void
*
*/
function plugin_user_create_moblog ($uid)
{
    global $_TABLES, $_MOBLOG_CONF;

    if (DB_count ($_TABLES['moblog'], 'uid', $uid) == 1) {
        return;
    }

    $c = DB_count($_TABLES['topics'], 'tid', 'General');
    if ($c) {
        $tid = 'General';
    } else {
        $result = DB_query("SELECT tid FROM {$_TABLES['topics']}");
        $A = DB_fetchArray($result);
        $tid = $A['tid'];
    }

    $sql = "INSERT INTO {$_TABLES['moblog']} "
         . "(uid, email, default_tid, default_subject, enable_thmb, thmb_width, thmb_height, image_width, image_height) "
         . "VALUES ('$uid', '', '$tid', 'Moblog', '1', '240', '240', '640', '640')";
    DB_query($sql);
}


?>