<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | Moblog Plugin                                                             |
// +---------------------------------------------------------------------------+
// | plugins/moblog/install_defaults.php                                       |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2009 by the following authors:                              |
// |                                                                           |
// | Authors: Yoshinori Tahara - taharaxp AT gmail DOT com                     |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+

if (strpos(strtolower($_SERVER['PHP_SELF']), 'install_defaults.php') !== false) {
    die('This file can not be used on its own!');
}

/*
 * Moblog default settings
 *
 * Initial Installation Defaults used when loading the online configuration
 * records. These settings are only used during the initial installation
 * and not referenced any more once the plugin is installed
 *
 */

global $_MOBLOG_DEFAULT;

$_MOBLOG_DEFAULT = array(
    'default_tid'         => 'General',
    'default_subject'     => 'Moblog',
    'enable_cronjob'      => '0',
    'image_width'         => '640',
    'image_height'        => '640',
    'image_maxbyte'       => '1000000', // 1000KB
    'enable_thmb'         => '1',
    'thmb_width'          => '240',
    'thmb_height'         => '240',
    'tmp_url'             => $_CONF['site_url'] . "/images/moblog/",
    'tmp_dir'             => $_CONF['path_html'] . "images/moblog/",
    'thumb_url'           => $_CONF['site_url'] . "/images/moblog/",
    'thumb_dir'           => $_CONF['path_html'] . "images/moblog/",
    'mailserver_host'     => 'localhost',
    'mailserver_user'     => 'admin',
    'mailserver_pass'     => 'password',
    'mailserver_port'     => '110',
);

/**
* Initialize Moblog plugin configuration
*
* Creates the database entries for the configuation if they don't already
* exist. Initial values will be taken from $_MOBLOG_CONF if available (e.g. from
* an old config.php), uses $_MOBLOG_DEFAULT otherwise.
*
* @return   boolean     true: success; false: an error occurred
*
*/
function plugin_initconfig_moblog()
{
    global $_MOBLOG_CONF, $_MOBLOG_DEFAULT;

    if (is_array($_MOBLOG_CONF) && (count($_MOBLOG_CONF) > 1)) {
        $_MOBLOG_DEFAULT = array_merge($_MOBLOG_DEFAULT, $_MOBLOG_CONF);
    }

    $c = config::get_instance();
    $n = 'moblog';
    $o = 0;
    if ($c->group_exists($n)) return true;
    $c->add('sg_main',             NULL,                                    'subgroup', 0, 0, NULL, 0,    true, $n);
    // ----------------------------------
    $c->add('fs_main',             NULL,                                    'fieldset', 0, 0, NULL, 0,    true, $n);
    $c->add('default_tid',         $_MOBLOG_DEFAULT['default_tid'],         'text',     0, 0, 0,    $o++, true, $n);
    $c->add('default_subject',     $_MOBLOG_DEFAULT['default_subject'],     'text',     0, 0, 0,    $o++, true, $n);
    $c->add('enable_cronjob',      $_MOBLOG_DEFAULT['enable_cronjob'],      'select',   0, 0, 0,    $o++, true, $n);
    // ----------------------------------
    $c->add('fs_images',           NULL,                                    'fieldset', 0, 1, NULL, 0,    true, $n);
    $c->add('image_width',         $_MOBLOG_DEFAULT['image_width'],         'text',     0, 1, 0,    $o++, true, $n);
    $c->add('image_height',        $_MOBLOG_DEFAULT['image_height'],        'text',     0, 1, 0,    $o++, true, $n);
    $c->add('image_maxbyte',       $_MOBLOG_DEFAULT['image_maxbyte'],       'text',     0, 1, 0,    $o++, true, $n);
    $c->add('enable_thmb',         $_MOBLOG_DEFAULT['enable_thmb'],         'select',   0, 1, 0,    $o++, true, $n);
    $c->add('thmb_width',          $_MOBLOG_DEFAULT['thmb_width'],          'text',     0, 1, 0,    $o++, true, $n);
    $c->add('thmb_height',         $_MOBLOG_DEFAULT['thmb_height'],         'text',     0, 1, 0,    $o++, true, $n);
    $c->add('tmp_url',             $_MOBLOG_DEFAULT['tmp_url'],             'text',     0, 1, 0,    $o++, true, $n);
    $c->add('tmp_dir',             $_MOBLOG_DEFAULT['tmp_dir'],             'text',     0, 1, 0,    $o++, true, $n);
    $c->add('thumb_url',           $_MOBLOG_DEFAULT['thumb_url'],           'text',     0, 1, 0,    $o++, true, $n);
    $c->add('thumb_dir',           $_MOBLOG_DEFAULT['thumb_dir'],           'text',     0, 1, 0,    $o++, true, $n);
    // ----------------------------------
    $c->add('fs_mailserver',       NULL,                                    'fieldset', 0, 2, NULL, 0,    true, $n);
    $c->add('mailserver_host',     $_MOBLOG_DEFAULT['mailserver_host'],     'text',     0, 2, 0,    $o++, true, $n);
    $c->add('mailserver_user',     $_MOBLOG_DEFAULT['mailserver_user'],     'text',     0, 2, 0,    $o++, true, $n);
    $c->add('mailserver_pass',     $_MOBLOG_DEFAULT['mailserver_pass'],     'text',     0, 2, 0,    $o++, true, $n);
    $c->add('mailserver_port',     $_MOBLOG_DEFAULT['mailserver_port'],     'text',     0, 2, 0,    $o++, true, $n);
    return true;
}

?>