<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | Moblog Plugin                                                             |
// +---------------------------------------------------------------------------+
// | plugins/moblog/moblog_story.php                                           |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2009 by the following authors:                              |
// |                                                                           |
// | Authors: Yoshinori Tahara (dengen) - taharaxp AT gmail DOT com            |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+

if (strpos(strtolower($_SERVER['PHP_SELF']), 'moblog_story.php') !== false) {
    die ('This file can not be used on its own.');
}

require_once $_CONF['path_system'] . 'classes/story.class.php';

class MoblogStory extends Story
{

    // Constructor
    function moblogStory() {
        parent::Story();
    }

    /**
     * Loads a story from the post data. This is the most exciting function in
     * the whole entire world. First it'll clean up that horrible Magic Quotes
     * crap. Then it'll do all Geeklog's funky security stuff, anti XSS, anti
     * SQL Injection. Yay.
     */
    function loadFromArgsArray(&$array)
    {
        global $_TABLES;

        /* magic_quotes_gpc cleanup routine now in submitstory() in
         * /public_html/admin/story.php
         */

        $retval = STORY_LOADED_OK; // default to success

        /* Load the trivial stuff: */
        $this->_loadBasics($array);

        /* Check to see if we have permission to edit this sid, and that this
         * sid is not a duplicate or anything horrible like that. ewww.
         */
        $sql
        = 'SELECT owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon ' . ' FROM ' . $_TABLES['stories']
            . ' WHERE sid=\'' . $this->_sid . '\'';
        $result = DB_query($sql);

        if ($result && (DB_numRows($result) > 0)) {
            /* Sid exists! Is it our article? */
            if ($this->_sid != $this->_originalSid) {
                // for story preview: don't abort
                $retval = STORY_DUPLICATE_SID;
            }

            $article = DB_fetchArray($result);
            /* Check Security */
            if (SEC_hasAccess($article['owner_id'], $article['group_id'],
                    $article['perm_owner'], $article['perm_group'],
                    $article['perm_members'], $article['perm_anon']) < 3) {
                return STORY_EXISTING_NO_EDIT_PERMISSION;
            }
        }

        // $this->_uid̃ANZX`FbN悤ɕύX
        $access = SEC_hasAccess_uid($this->_owner_id, $this->_group_id, $this->_perm_owner, $this->_perm_group,
                                    $this->_perm_members, $this->_perm_anon, $this->_uid);

        // $this->_uid̃ANZX`FbN悤ɕύX
        if (($access < 3) || !SEC_hasTopicAccess($this->_tid) || !SEC_inGroup($this->_group_id, $this->_uid)) {
            return STORY_NO_ACCESS_PARAMS;
        }

        /* Load up the topic name and icon */
        $topic = DB_query("SELECT topic, imageurl FROM {$_TABLES['topics']} WHERE tid='{$this->_tid}'");
        $topic = DB_fetchArray($topic);
        $this->_topic = $topic['topic'];
        $this->_imageurl = $topic['imageurl'];

        //$title = COM_stripSlashes( $array['title'] );
        //$intro = COM_stripSlashes( $array['introtext'] );
        //$body = COM_stripSlashes( $array['bodytext'] );

        /* Then load the title, intro and body */
        if (($array['postmode'] == 'html') || ($array['postmode'] == 'adveditor') || ($array['postmode'] == 'wikitext')) {
            $this->_htmlLoadStory($array['title'], $array['introtext'], $array['bodytext']);

            if ($this->_postmode == 'adveditor') {
                $this->_advanced_editor_mode = 1;
                $this->_postmode = 'html';
            } else {
                $this->_advanced_editor_mode = 0;
            }
        } else {
            $this->_advanced_editor_mode = 0;
            $this->_plainTextLoadStory($array['title'], $array['introtext'], $array['bodytext']);
        }

        if (empty($this->_title) || empty($this->_introtext)) {
            return STORY_EMPTY_REQUIRED_FIELDS;
        }

        $this->_sanitizeData();

        return $retval;
    }


    /**
     * Loads a submitted story from postdata
     */
    // $arrayp[^ǉ
    function loadSubmission(&$array=array())
    {
        if (empty($array)) {
            $array = $_POST;
        }

        $this->_expire = time();
        $this->_date = time();
        $this->_expiredate = 0;

        // Handle Magic GPC Garbage:
        while (list($key, $value) = each($array))
        {
            $array[$key] = COM_stripslashes($value);
        }

        $this->_postmode = COM_applyFilter($array['postmode']);
        $this->_sid = COM_applyFilter($array['sid']);
        $this->_uid = COM_applyFilter($array['uid'], true);
        if ($this->_uid < 1) {
            $this->_uid = 1;
        }
        $this->_unixdate = COM_applyFilter($array['date'], true);

        if (!isset($array['bodytext'])) {
            $array['bodytext'] = '';
        }

        /* Then load the title, intro and body */
        if (($array['postmode'] == 'html') || ($array['postmode'] == 'adveditor')) {
            $this->_htmlLoadStory($array['title'], $array['introtext'], $array['bodytext']);

            if ($this->_postmode == 'adveditor') {
                $this->_advanced_editor_mode = 1;
                $this->_postmode = 'html';
            } else {
                $this->_advanced_editor_mode = 0;
            }
        } else {
            $this->_advanced_editor_mode = 0;
            $this->_plainTextLoadStory($array['title'], $array['introtext'], $array['bodytext']);
        }

        $this->_tid = COM_applyFilter($array['tid']);

        if (empty($this->_title) || empty($this->_introtext)) {
            return STORY_EMPTY_REQUIRED_FIELDS;
        }

        return STORY_LOADED_OK;
    }


    /**
     * This is the importantest bit. This function must load the title, intro
     * and body of the article from the post array, providing all appropriate
     * conversions of HTML mode content into the nice safe form that geeklog
     * can then (simply) spit back out into the page on render. After doing a
     * magic tags replacement.
     *
     * This DOES NOT ADDSLASHES! We do that on DB store, because we want to
     * keep our internal variables in "display mode", not in db mode or anything.
     *
     * @param $title    string  posttitle, only had stripslashes if necessary
     * @param $intro    string  introtext, only had stripslashes if necessary
     * @param $body     string   bodytext, only had stripslashes if necessary
     * @return nothing
     * @access private
     */
    // $this->_uid̃ANZX`FbN悤ɕύX
    function _htmlLoadStory($title, $intro, $body)
    {
        global $_CONF;

        // fix for bug in advanced editor
        if ($_CONF['advanced_editor'] && ($body == '<br' . XHTML . '>')) {
            $body = '';
        }

        $this->_title = htmlspecialchars(strip_tags(COM_checkWords($title)));
        $this->_introtext = COM_checkHTML_uid(COM_checkWords($intro), 'story.edit', $this->_uid);
        $this->_bodytext = COM_checkHTML_uid(COM_checkWords($body), 'story.edit', $this->_uid);
    }

    /**
     * Saves a story submission.
     *
     * @return  integer result code explaining behaviour.
     */
    // OɃZbgꂽ$this->_uidŃf[^ۑ悤ɂ
    // OɃZbgꂽ$this->_sidŃf[^ۑ悤ɂ
    // 'story.submit'̌`FbNȂ悤ɕύXiOItłf[^ۑł悤ɂj
    // Ƀp[^$post_directlyǉ
    function saveSubmission($post_directly = '')
    {
        global $_USER, $_CONF, $_TABLES;

        if (empty($this->_sid)) $this->_sid = COM_makeSid();

        /*
        if (COM_isAnonUser()) {
            $this->_uid = 1;
        } else {
            $this->_uid = $_USER['uid'];
        }
        */

        $tmptid = addslashes(COM_sanitizeID($this->_tid));

        $result = DB_query('SELECT group_id,perm_owner,perm_group,perm_members,perm_anon FROM ' .
                            "{$_TABLES['topics']} WHERE tid = '{$tmptid}'" .
                            COM_getTopicSQL('AND', $this->_uid));

        if (DB_numRows($result) == 0) {
            // user doesn't have access to this topic - bail
            return STORY_NO_ACCESS_TOPIC;
        }

        $T = DB_fetchArray($result);
//        if (($_CONF['storysubmission'] == 1) && !SEC_hasRights_uid('story.submit', 'AND', $this->_uid)) {
        if (($_CONF['storysubmission'] == 1) && (empty($post_directly) || $post_directly === false)) {
            $this->_sid = addslashes($this->_sid);
            $this->_tid = $tmptid;
            $this->_title = addslashes($this->_title);
            $this->_introtext = addslashes($this->_introtext);
            $this->_bodytext = addslashes($this->_bodytext);
            $this->_postmode = addslashes($this->_postmode);
            DB_save($_TABLES['storysubmission'], 'sid,tid,uid,title,introtext,bodytext,date,postmode',
                        "'{$this->_sid}','{$this->_tid}',{$this->_uid},'{$this->_title}'," .
                        "'{$this->_introtext}','{$this->_bodytext}',NOW(),'{$this->_postmode}'");

            return STORY_SAVED_SUBMISSION;
        } else {
            // post this story directly. First establish the necessary missing data.
            $this->_sanitizeData();

            if (!isset($_CONF['show_topic_icon'])) {
                $_CONF['show_topic_icon'] = 1;
            }

            if (DB_getItem($_TABLES['topics'], 'archive_flag', "tid = '{$tmptid}'") == 1) {
                $this->_frontpage = 0;
            } elseif (isset($_CONF['frontpage'])) {
                $this->_frontpage = $_CONF['frontpage'];
            } else {
                $this->_frontpage = 1;
            }

            $this->_oldsid = $this->_sid;
            $this->_date = mktime();
            $this->_featured = 0;
            $this->_commentcode = $_CONF['comment_code'];
            $this->_comment_expire = 0; // ǉ
            $this->_trackbackcode = $_CONF['trackback_code'];
            $this->_statuscode = 0;
            $this->_show_topic_icon = $_CONF['show_topic_icon'];
            /*
            if (COM_isAnonUser()) {
                $this->_owner_id = 1;
            } else {
                $this->_owner_id = $_USER['uid'];
            }
            */
            $this->_owner_id = $this->_uid;

            $this->_group_id = $T['group_id'];
            $this->_perm_owner = $T['perm_owner'];
            $this->_perm_group = $T['perm_group'];
            $this->_perm_members = $T['perm_members'];
            $this->_perm_anon = $T['perm_anon'];

            $this->saveToDatabase();

            PLG_itemSaved($this->_sid, 'article');
            COM_rdfUpToDateCheck();
            COM_olderStuff();

            return STORY_SAVED;
        }
    }
}

/**
* Checks if current user has access to the given object
*
* This function takes the access info from a Geeklog object
* and let's us know if they have access to the object
* returns 3 for read/edit, 2 for read only and 0 for no
* access
*
* @param        int     $owner_id       ID of the owner of object
* @param        int     $group_id       ID of group object belongs to
* @param        int     $perm_owner     Permissions the owner has
* @param        int     $perm_group     Permissions the gorup has
* @param        int     $perm_members   Permissions logged in members have
* @param        int     $perm_anon      Permissions anonymous users have
* @return       int 	returns 3 for read/edit 2 for read only 0 for no access
*
*/
// $uidp[^œn悤ɕύX
function SEC_hasAccess_uid($owner_id,$group_id,$perm_owner,$perm_group,$perm_members,$perm_anon, $uid='')
{
    global $_USER;

    if (empty($uid)) {
        // Cache current user id
        if (empty ($_USER['uid'])) {
            $uid = 1;
        } else {
            $uid = $_USER['uid'];
        }
    }

    // If user is in Root group then return full access
    if (SEC_inGroup('Root', $uid)) {
        return 3;
    }

    // If user is owner then return 1 now
    if ($uid == $owner_id) return $perm_owner;

    // Not private, if user is in group then give access
    if (SEC_inGroup($group_id, $uid)) {
        return $perm_group;
    } else {
        if ($uid == 1) {
            // This is an anonymous user, return it's rights
            return $perm_anon;
        } else {
            // This is a logged in member, return their rights
            return $perm_members;
        }
    }
}

/**
* This function checks html tags.
*
* Checks to see that the HTML tags are on the approved list and
* removes them if not.
*
* @param    string  $str            HTML to check
* @param    string  $permissions    comma-separated list of rights which identify the current user as an "Admin"
* @return   string                  Filtered HTML
*
*/
// $uidp[^œn悤ɕύX
function COM_checkHTML_uid( $str, $permissions = 'story.edit', $uid = '' )
{
    global $_CONF;

    // replace any \ with &#092; (HTML equiv)
    $str = str_replace('\\', '&#092;', COM_stripslashes($str) );

    // Get rid of any newline characters
    $str = preg_replace( "/\n/", '', $str );

    // Replace any $ with &#36; (HTML equiv)
    $str = str_replace( '$', '&#36;', $str );
    // handle [code] ... [/code]
    do
    {
        $start_pos = MBYTE_strpos( MBYTE_strtolower( $str ), '[code]' );
        if( $start_pos !== false )
        {
            $end_pos = MBYTE_strpos( MBYTE_strtolower( $str ), '[/code]' );
            if( $end_pos !== false )
            {
                $encoded = COM_handleCode( MBYTE_substr( $str, $start_pos + 6,
                        $end_pos - ( $start_pos + 6 )));
                $encoded = '<pre><code>' . $encoded . '</code></pre>';
                $str = MBYTE_substr( $str, 0, $start_pos ) . $encoded
                     . MBYTE_substr( $str, $end_pos + 7 );
            }
            else // missing [/code]
            {
                // Treat the rest of the text as code (so as not to lose any
                // special characters). However, the calling entity should
                // better be checking for missing [/code] before calling this
                // function ...
                $encoded = COM_handleCode( MBYTE_substr( $str, $start_pos + 6 ));
                $encoded = '<pre><code>' . $encoded . '</code></pre>';
                $str = MBYTE_substr( $str, 0, $start_pos ) . $encoded;
            }
        }
    }
    while( $start_pos !== false );

    // handle [raw] ... [/raw]
    do
    {
        $start_pos = MBYTE_strpos( MBYTE_strtolower( $str ), '[raw]' );
        if( $start_pos !== false )
        {
            $end_pos = MBYTE_strpos( MBYTE_strtolower( $str ), '[/raw]' );
            if( $end_pos !== false )
            {
                $encoded = COM_handleCode( MBYTE_substr( $str, $start_pos + 5,
                        $end_pos - ( $start_pos + 5 )));
                // [raw2] to avoid infinite loop. Not HTML comment as we strip
                // them later.
                $encoded = '[raw2]' . $encoded . '[/raw2]';
                $str = MBYTE_substr( $str, 0, $start_pos ) . $encoded
                     . MBYTE_substr( $str, $end_pos + 6 );
            }
            else // missing [/raw]
            {
                // Treat the rest of the text as raw (so as not to lose any
                // special characters). However, the calling entity should
                // better be checking for missing [/raw] before calling this
                // function ...
                $encoded = COM_handleCode( MBYTE_substr( $str, $start_pos + 5 ));
                // [raw2] to avoid infinite loop. Not HTML comment as we strip
                // them later.
                $encoded = '[raw2]' . $encoded . '[/raw2]';
                $str = MBYTE_substr( $str, 0, $start_pos ) . $encoded;
            }
        }
    }
    while( $start_pos !== false );

    if( isset( $_CONF['skip_html_filter_for_root'] ) &&
             ( $_CONF['skip_html_filter_for_root'] == 1 ) &&
            SEC_inGroup( 'Root', $uid ))
    {
        return $str;
    }

    // strip_tags() gets confused by HTML comments ...
    $str = preg_replace( '/<!--.+?-->/', '', $str );

    $filter = new kses4;
    if( isset( $_CONF['allowed_protocols'] ) && is_array( $_CONF['allowed_protocols'] ) && ( sizeof( $_CONF['allowed_protocols'] ) > 0 ))
    {
        $filter->SetProtocols( $_CONF['allowed_protocols'] );
    }
    else
    {
        $filter->SetProtocols( array( 'http:', 'https:', 'ftp:' ));
    }

    if( empty( $permissions) || !SEC_hasRights_uid( $permissions, 'AND', $uid ) ||
            empty( $_CONF['admin_html'] ))
    {
        $html = $_CONF['user_html'];
    }
    else
    {
        if ($_CONF['advanced_editor'] && is_array($_CONF['advanced_html'])) {
            $html = array_merge_recursive( $_CONF['user_html'],
                                           $_CONF['admin_html'],
                                           $_CONF['advanced_html'] );
        } else {
            $html = array_merge_recursive( $_CONF['user_html'],
                                           $_CONF['admin_html'] );
        }
    }

    foreach( $html as $tag => $attr )
    {
        $filter->AddHTML( $tag, $attr );
    }
    /* Replace [raw][/raw] with <!--raw--><!--/raw-->, note done "late" because
     * of the above noted // strip_tags() gets confused by HTML comments ...
     */
    $str = $filter->Parse( $str );
    $str = str_replace('[raw2]','<!--raw--><span class="raw">', $str);
    $str = str_replace('[/raw2]','</span><!--/raw-->', $str);

    return $str;
}

/**
* Checks if current user has rights to a feature
*
* Takes either a single feature or an array of features and returns
* an array of whether the user has those rights
*
* @param        string|array        $features       Features to check
* @param        string              $operator       Either 'and' or 'or'. Default is 'and'.  Used if checking more than one feature.
* @return       boolean     Return true if current user has access to feature(s), otherwise false.
*
*/
// $uidp[^œn悤ɕύX
function SEC_hasRights_uid($features,$operator='AND',$uid='')
{
    global $_USER, $_SEC_VERBOSE;

    $_RIGHTS = explode(',', SEC_getUserPermissions('', $uid));

    if (strstr($features,',')) {
        $features = explode(',',$features);
    }

    if (is_array($features)) {
        // check all values passed
        for ($i = 0; $i < count($features); $i++) {
            if ($operator == 'OR') {
                // OR operator, return as soon as we find a true one
                if (in_array($features[$i],$_RIGHTS)) {
                    if ($_SEC_VERBOSE) {
                        COM_errorLog('SECURITY: user has access to ' . $features[$i],1);
                    }
                    return true;
                }
            } else {
                // this is an "AND" operator, bail if we find a false one
                if (!in_array($features[$i],$_RIGHTS)) {
                    if ($_SEC_VERBOSE) {
                        COM_errorLog('SECURITY: user does not have access to ' . $features[$i],1);
                    }
                    return false;
                }
            }
        }

        if ($operator == 'OR') {
            if ($_SEC_VERBOSE) {
                COM_errorLog('SECURITY: user does not have access to ' . $features[$i],1);
            }
            return false;
                } else {
            if ($_SEC_VERBOSE) {
                COM_errorLog('SECURITY: user has access to ' . $features[$i],1);
            }
            return true;
        }
    } else {
        // Check the one value
        if ($_SEC_VERBOSE) {
            if (in_array($features,$_RIGHTS)) {
                COM_errorLog('SECURITY: user has access to ' . $features,1);
            } else {
                COM_errorLog('SECURITY: user does not have access to ' . $features,1);
            }
        }
        return in_array($features,$_RIGHTS);
    }
}


/**
 * Submit a new or updated story.
 *
 * @param   array   args    Contains all the data provided by the client
 */
function MOBLOG_submitStory($args)
{
    global $_CONF, $_USER, $_TABLES, $LANG24, $MESSAGE;

    if (!SEC_hasRights_uid('story.edit', 'AND', $args['uid'])) {
        return PLG_RET_AUTH_FAILED;
    }

    require_once $_CONF['path_system'] . 'lib-comment.php';

    if(empty($args['owner_id'])) {
        $args['owner_id'] = $args['uid'];
    }

    if (empty($args['group_id'])) {
        $args['group_id'] = SEC_getFeatureGroup('story.edit', $args['uid']);
    }

    if (empty($args['postmode'])) {
        $args['postmode'] = 'html';
    }

    if (!isset($args['perm_owner'])) {
        $args['perm_owner'] = $_CONF['default_permissions_story'][0];
    } else {
        $args['perm_owner'] = COM_applyBasicFilter($args['perm_owner'], true);
    }

    if (!isset($args['perm_group'])) {
        $args['perm_group'] = $_CONF['default_permissions_story'][1];
    } else {
        $args['perm_group'] = COM_applyBasicFilter($args['perm_group'], true);
    }

    if (!isset($args['perm_members'])) {
        $args['perm_members'] = $_CONF['default_permissions_story'][2];
    } else {
        $args['perm_members'] = COM_applyBasicFilter($args['perm_members'], true);
    }

    if (!isset($args['perm_anon'])) {
        $args['perm_anon'] = $_CONF['default_permissions_story'][3];
    } else {
        $args['perm_anon'] = COM_applyBasicFilter($args['perm_anon'], true);
    }
    
    if (!isset($args['draft_flag'])) {
        $args['draft_flag'] = $_CONF['draft_flag'];
    }

    if (empty($args['frontpage'])) {
        $args['frontpage'] = $_CONF['frontpage'];
    }

    if (empty($args['show_topic_icon'])) {
        $args['show_topic_icon'] = $_CONF['show_topic_icon'];
    }

    if (!isset($args['sid'])) {
        $args['sid'] = '';
    }
    $args['sid'] = COM_sanitizeID($args['sid']);
    if (strlen($args['sid']) > STORY_MAX_ID_LENGTH) {
        $args['sid'] = WS_makeId($args['slug'], STORY_MAX_ID_LENGTH);
    }

    $story = new MoblogStory();
    // This function is also doing the security checks
    $result = $story->loadFromArgsArray($args);
    $sid = $story->getSid();
    switch ($result) {
        case STORY_DUPLICATE_SID:
            COM_errorLog($LANG24[24], 1);
            return PLG_RET_ERROR;

        case STORY_EXISTING_NO_EDIT_PERMISSION:
            $un = DB_getItem($_TABLES['users'], 'username', "uid = '{$args['uid']}'");
            COM_accessLog("User $un tried to illegally submit or edit story $sid.");
            return PLG_RET_PERMISSION_DENIED;

        case STORY_NO_ACCESS_PARAMS:
            $un = DB_getItem($_TABLES['users'], 'username', "uid = '{$args['uid']}'");
            COM_accessLog("User $un tried to illegally submit or edit story $sid.");
            return PLG_RET_PERMISSION_DENIED;

        case STORY_EMPTY_REQUIRED_FIELDS:
            COM_errorLog($LANG24[31], 1);
            return PLG_RET_ERROR;

        default:
            break;
    }

    $result = $story->saveToDatabase();
    if ($result == STORY_SAVED) {
        // see if any plugins want to act on that story
        PLG_itemSaved($sid, 'article');

        // update feed(s) and Older Stories block
        COM_rdfUpToDateCheck('article', $story->DisplayElements('tid'), $sid);
        COM_olderStuff();
        CMT_updateCommentcodes();
        return PLG_RET_OK;
    }
    return PLG_RET_ERROR;
}

/**
* Saves a story submission
*
* @param    array   $args  Data for that submission
* @return   string         HTML redirect
*
*/
function MOBLOG_savestory ($args, $post_directly='')
{
    global $_CONF;

    $story = new MoblogStory('submission');
    $story->loadSubmission($args);

    // pseudo-formatted story text for the spam check
    $result = PLG_checkforSpam ($story->GetSpamCheckFormat(), $_CONF['spamx']);
    if ($result > 0) {
        COM_displayMessageAndAbort ($result, 'spamx', 403, 'Forbidden');
    }

    $result = $story->saveSubmission($post_directly);

    return $result;
}

?>
